@@ -38,17 +38,54 @@ use 5.006;
 use strict;
 use warnings FATAL => 'all';
 
+use Exporter 'import';
+
+
+
 =head1 NAME
 
 App::Dochazka::CLI - Dochazka command line client
 
+
+
 =head1 VERSION
 
-Version 0.064
+Version 0.096
 
 =cut
 
-our $VERSION = '0.064';
+our $VERSION = '0.096';
+
+
+
+=head1 PACKAGE VARIABLES AND EXPORTS
+
+=over
+
+=item * C<< $current_date >>
+
+The current date, as determined by C<bin/dochazka-cli>
+
+=item * C<< $current_emp >>
+
+The L<App::Dochazka::Model::Employee> object of the current employee.
+
+=item * C<< $current_priv >>
+
+The privlevel of the current employee.
+
+=item * C<< $message >>
+
+A package variable to hold the message to be displayed to the user when she
+enters a command.
+
+=back
+
+=cut
+
+our ( $current_date, $current_emp, $current_priv, $message );
+our @EXPORT_OK = qw( $current_date $current_emp $current_priv $message );
+
 
 
 =head1 SYNOPSIS
