@@ -38,8 +38,11 @@ use 5.012;
 use strict;
 use warnings;
 
-use App::CELL qw( $CELL );
-use App::Dochazka::CLI::HTTP qw( send_req );
+use App::CELL qw( $CELL $log );
+use App::Dochazka::CLI::Help qw( no_help );
+use App::Dochazka::CLI::Set qw( set_employee );
+use App::Dochazka::CLI::Show qw( show_employee );
+use Web::MREST::CLI::UserAgent qw( send_req );
 use Test::Deep::NoTest;
 
 =head1 NAME
@@ -51,11 +54,11 @@ App::Dochazka::CLI::Parser - Parser for Dochazka command line client
 
 =head1 VERSION
 
-Version 0.064
+Version 0.096
 
 =cut
 
-our $VERSION = '0.064';
+our $VERSION = '0.096';
 our $anything = qr/^.+$/i;
 
 
@@ -90,56 +93,109 @@ three characters.
 
 =cut
 
-my $method; # store the HTTP method in a package variable so it is remembered
-
 sub parse_tokens {
     my ( $pre, $tokens ) = @_; 
-    return $CELL->status_err( "No more tokens" ) unless ref $tokens;
+    return $CELL->status_err( "No more tokens" ) unless ref( $tokens );
     my @tokens = @$tokens;
     my $token = shift @tokens;
 
-    # the first token designates the HTTP method
-    if ( @$pre == 0 ) { # first token is supposed to be the HTTP method
+    if ( @$pre == 0 ) { 
 
-        # GET ''
         if ( $token =~  m/^GET/i ) {
-            $method = 'GET';
+        # GET
             parse_tokens( [ 'GET' ], \@tokens ) if @tokens;
             die send_req( 'GET', '' );
         }
-
-        # PUT ''
         elsif ( $token =~ m/^PUT/i ) {
-            $method = 'PUT';
+        # PUT
             parse_tokens( [ 'PUT' ], \@tokens ) if @tokens;
             die send_req( 'PUT', '' );
         } 
-        
-        # POST ''
         elsif ( $token =~ m/^POS/i ) {
-            $method = 'POST';
+        # POST 
             parse_tokens( [ 'POST' ], \@tokens ) if @tokens;
             die send_req( 'POST', '' );
         } 
-        
-        # DELETE ''
         elsif ( $token =~ m/^DEL/i ) {
-            $method = 'DELETE';
+        # DELETE ''
             parse_tokens( [ 'DELETE' ], \@tokens ) if @tokens;
             die send_req( 'DELETE', '' );
         }
-
+        elsif ( $token =~ m/^SHO/i ) {
+        # SHOW
+            parse_tokens( [ 'SHOW' ], \@tokens ) if @tokens;
+            die no_help( $token, @tokens );
+        }
+        elsif ( $token =~ m/^SET/i ) {
+        # SET
+            parse_tokens( [ 'SET' ], \@tokens ) if @tokens;
+            die no_help( $token, @tokens );
+        }
+        elsif ( $token =~ m/^(exi)|(qu)|(\\q)/i ) { 
         # EXIT, QUIT, and the like
-        elsif ( $token =~ m/^(exi)|(qu)|(\\q)/i and eq_deeply( $pre, [] ) ) { 
             die $CELL->status_ok( 'DOCHAZKA_CLI_EXIT' );
         }   
-
         die $CELL->status_err( 'DOCHAZKA_CLI_PARSE_ERROR' );
     }
+    elsif ( @$pre >= 1 ) { 
+
+        my $t1 = $pre->[0];
+
+        if ( $t1 =~ m/^(GET)|(POST)|(PUT)|(DELETE)$/ ) {
+            http_test_and_die( $pre, $token, @tokens );
+            die "We were supposed to die in http_test_and_die, but we didn't!";
+        } 
+        elsif ( $t1 eq 'SHOW' ) {
+
+            #print "The method is SHOW\n";
+
+            if ( $token =~ m/^EMP/i ) {
+            # SHOW EMPLOYEE [$KEY]
+                die show_employee( @tokens );
+            }
+
+        } elsif ( $t1 eq 'SET' ) {
+
+            #print "The method is SET\n";
+
+            if ( $token =~ m/^EMP/i ) {
+            # SET EMPLOYEE
+                die set_employee( @tokens );
+            }
+
+        }
+
+    }
+    # we have gone all the way through the state machine without a match
+    die $CELL->status_err( 'DOCHAZKA_CLI_PARSE_ERROR' );
+
+}
+
+
+=head2 http_test_and_die
+
+When the first token is GET, POST, PUT, or DELETE, we enter the "HTTP testing"
+branch of the parser, which is implemented by this routine.
+
+As the name indicates, the routine should not return.
+
+=cut
+
+sub http_test_and_die {
+    my ( $pre, $token, @tokens ) = @_;
+    my $method = $pre->[0];
+
+    #print "http_test_and_die, method is $method, pre is " . @$pre . "\n";
 
-    # second token represents the resource class ( top-level, employee, priv, etc.)
     if ( @$pre == 1 ) {
 
+        #
+        # root resource
+        #
+        if ( $token eq '/' ) {
+            die send_req( $method, '/' );
+        }
+
         #
         # activity resource: recurse
         #
@@ -206,17 +262,22 @@ sub parse_tokens {
             die send_req( $method, 'dbstatus' );
         }
 
-        # "/docu $RESOURCE"
+        # "/docu/{pod,html,text} \"$RESOURCE'""
         if ( $token =~ m/^doc/i ) { 
-            if ( @tokens ) {
-                if ( $tokens[0] =~ m/^htm/i ) {
-                    my $resource = join(' ', @tokens[1..$#tokens]);
-                    $resource = '"' . $resource . '"' unless $resource =~ m/^\".*\"$/;
-                    die send_req( $method, 'docu/html', "{ \"resource\" : $resource }" );
+            if ( scalar( @tokens ) == 2 ) {
+                my $pod_html = shift @tokens;
+                my $json = join( ' ', @tokens );
+                if ( $json ) {
+                    $json = '"' . $json . '"' unless $json =~ m/^".*"$/;
+                    if ( $pod_html =~ m/^pod/i ) {
+                        die send_req( $method, 'docu/pod', $json );
+                    } elsif ( $pod_html =~ m/^htm/i ) {
+                        die send_req( $method, 'docu/html', $json );
+                    } elsif ( $pod_html =~ m/^tex/i ) {
+                        die send_req( $method, 'docu/text', $json );
+                    } 
                 } else {
-                    my $resource = join(' ', @tokens);
-                    $resource = '"' . $resource . '"' unless $resource =~ m/^\".*\"$/;
-                    die send_req( $method, 'docu', "{ \"resource\" : $resource }" );
+                    print "You should specify a resource\n";
                 }
             } else {
                 die send_req( $method, 'docu' );
@@ -228,6 +289,12 @@ sub parse_tokens {
             die send_req( $method, 'echo', join(' ', @tokens) );
         }   
 
+        # "/employee..."
+        if ( $token =~ m/^emp/i ) {
+            parse_tokens( [ $method, 'EMPLOYEE' ], \@tokens ) if @tokens;
+            die send_req( $method, 'employee' );
+        }
+
         # "/forbidden"
         if ( $token =~ m/^for/i ) {
             die send_req( $method, "forbidden" );
@@ -235,28 +302,23 @@ sub parse_tokens {
 
         # "/help"
         if ( $token =~ m/^hel/i ) {
-            die send_req( $method, "help" );
+            die send_req( $method, "help", join( ' ', @tokens ) );
         }
     
-        # "/metaparam $JSON"
-        # "/metaparam/:param"
-        if ( $token =~ m/^met/i ) {
-            if ( @tokens ) {
-                if ( $method =~ m/^(GET)|(PUT)|(DELETE)$/ ) {
-                    die send_req( $method, "metaparam/$tokens[0]" );
-                }
-                if ( $tokens[0] =~ m/^{/ ) {
-                    die send_req( $method, "metaparam", join(' ', @tokens) );
-                } 
-                my $new_value = join(' ', @tokens[1..$#tokens]);
-                $new_value = '"' . $new_value . '"' unless $new_value =~ m/^({)|(\[)/;
-                die send_req( $method, "metaparam", "{ \"name\" : \"$tokens[0]\", \"value\" : $new_value }" );
+        # "/param $TYPE $JSON"
+        # "/param/:type/:param"
+        if ( $token =~ m/^par/i ) {
+            if ( scalar( @tokens ) > 1 ) {
+                my $type = $tokens[0];
+                my $param = $tokens[1];
+                my $json = join( ' ', @tokens[2..$#tokens] );
+                die send_req( $method, "param/$type/$param", $json );
             }
         }
     
-        # "/not_implemented"
-        if ( $token =~ m/^not/i ) {
-            die send_req( $method, "not_implemented" );
+        # "/noop"
+        if ( $token =~ m/^noo/i ) {
+            die send_req( $method, "noop" );
         }
 
         # "/session"
@@ -264,13 +326,18 @@ sub parse_tokens {
             die send_req( $method, "session" );
         }
 
-        # "/siteparam/:param"
-        if ( $token =~ m/^sit/i ) {
+        # "/test/?:specs"
+        if ( $token =~ m/^tes/i ) {
+            my $uri_path;
             if ( @tokens ) {
-                die send_req( $method, "siteparam/$tokens[0]" );
+                my $uri_ext .= join( ' ', @tokens ) if @tokens;
+                $uri_path = "test/$uri_ext";
+            } else {
+                $uri_path = "test";
             }
+            die send_req( $method, $uri_path );
         }
-    
+
         # "/version"
         if ( $token =~ m/^ver/i and eq_deeply( $pre, [ $method ] ) ) {
             die send_req( $method, 'version' );
@@ -351,6 +418,14 @@ sub parse_tokens {
             }
         }
 
+        # "/interval/summary/?:qualifiers"
+        if ( $token =~ m/^sum/ ) {
+            if ( @tokens ) {
+                die send_req( $method, 'interval/summary/' . join( ' ', @tokens ) );
+            }
+            die send_req( $method, "interval/summary" );
+        }
+
     }
 
 
@@ -444,11 +519,12 @@ sub parse_tokens {
 
         # "/schedule/eid/:eid/?:ts"
         if ( $token =~ m/^eid/i ) {
-            if ( $tokens[1] ) {
-                die send_req( $method, "schedule/eid/$tokens[0]/$tokens[1]" );
-            } 
-            if ( $tokens[0] ) {
-                die send_req( $method, "schedule/eid/$tokens[0]" );
+            if ( @tokens ) {
+                my $urip = "schedule/eid/$tokens[0]";
+                if ( defined $tokens[1] ) {
+                    $urip .= "/" . join( ' ', @tokens[1..$#tokens] );
+                }
+                die send_req( $method, $urip );
             }
         }
         
@@ -520,21 +596,22 @@ sub parse_tokens {
 
         # "/schedule/nick/:nick/?:ts"
         if ( $token =~ m/^nic/i ) {
-            if ( $tokens[1] ) {
-                die send_req( $method, "schedule/nick/$tokens[0]/$tokens[1]" );
-            } 
-            if ( $tokens[0] ) {
-                die send_req( $method, "schedule/nick/$tokens[0]" );
+            if ( @tokens ) {
+                my $urip = "schedule/nick/$tokens[0]";
+                if ( defined $tokens[1] ) {
+                    $urip .= "/" . join( ' ', @tokens[1..$#tokens] );
+                }
+                die send_req( $method, $urip );
             }
         }
 
         # "/schedule/self/?:ts"
         if ( $token =~ m/^sel/i ) {
-            if ( not @tokens ) {
-                die send_req( $method, "schedule/self" );
-            } else {
-                die send_req( $method, "schedule/self/$tokens[0]" );
+            my $urip = "schedule/self";
+            if ( @tokens ) {
+                $urip .= "/" . join( ' ', @tokens );
             }
+            die send_req( $method, $urip );
         }
 
         # "/schedule/sid/:sid"
@@ -614,8 +691,10 @@ sub parse_tokens {
     #
     # employee resource handlers
     #
-    elsif ( exists $pre->[1] and $pre->[1] eq 'EMPLOYEE' ) {
-        
+    if ( exists $pre->[1] and $pre->[1] eq 'EMPLOYEE' ) {
+
+        #print "EMPLOYEE\n";
+
         # "/employee/count"
         # "/employee/count/:priv"
         if ( $token =~ m/^cou/i ) {
@@ -627,9 +706,21 @@ sub parse_tokens {
 
         # "/employee/current"
         # "/employee/current/priv"
+        if ( $token =~ m/^cur/i ) {
+            if ( @tokens ) {
+                if ( $tokens[0] =~ m/^pri/i ) {
+                    die send_req( $method, 'employee/current/priv' );
+                } elsif ( $method ne 'GET' ) {
+                    die send_req( $method, 'employee/current', join(' ', @tokens ) );
+                }
+            } else {
+                die send_req( $method, 'employee/current' );
+            }
+        }
+
         # "/employee/self"
         # "/employee/self/priv"
-        if ( $token =~ m/^sel/i or $token =~ m/^cur/i ) {
+        if ( $token =~ m/^sel/i ) {
             if ( @tokens ) {
                 if ( $tokens[0] =~ m/^pri/i ) {
                     die send_req( $method, 'employee/self/priv' );
@@ -644,6 +735,7 @@ sub parse_tokens {
         # "/employee/eid [$JSON]"
         # "/employee/eid/:eid [$JSON]"
         if ( $token =~ m/^eid/i ) {
+            print "employee eid\n";
             if ( @tokens ) {
                 if ( $tokens[0] =~ m/^\d+/ ) {
                     my $eid = $tokens[0];
@@ -671,6 +763,18 @@ sub parse_tokens {
                 }
             }
         }
+
+        # "/employee/search/nick/:key"
+        if ( $token =~ m/^sea/i ) {
+            my $token = shift @tokens;
+            if ( $token =~ m/^nic/i ) {
+                my $key = shift @tokens;
+                if ( $key ) {
+                    die send_req( $method, "employee/search/nick/$key" );
+                }
+            }
+            die send_req( $method, "employee/search" );
+        }
     }   
 
     #
@@ -686,20 +790,21 @@ sub parse_tokens {
     
         # "/priv/self/?:ts"
         if ( $token =~ m/^sel/i ) {
-            if ( not @tokens ) {
-                die send_req( $method, "priv/self" );
-            } else {
-                die send_req( $method, "priv/self/$tokens[0]" );
+            my $urip = "schedule/self";
+            if ( @tokens ) {
+                $urip .= "/" . join( ' ', @tokens );
             }
+            die send_req( $method, $urip );
         }
 
         # "/priv/eid/:eid/?:ts"
         if ( $token =~ m/^eid/i ) {
-            if ( $tokens[1] ) {
-                die send_req( $method, "priv/eid/$tokens[0]/$tokens[1]" );
-            } 
-            if ( $tokens[0] ) {
-                die send_req( $method, "priv/eid/$tokens[0]" );
+            if ( @tokens ) {
+                my $urip = "priv/eid/$tokens[0]";
+                if ( defined $tokens[1] ) {
+                    $urip .= "/" . join( ' ', @tokens[1..$#tokens] );
+                }
+                die send_req( $method, $urip );
             }
         }
         
@@ -718,20 +823,29 @@ sub parse_tokens {
 
             # "/priv/history/eid/:eid [$JSON]"
             # "/priv/history/eid/:eid/:tsrange"
-            if ( $tokens[0] and $tokens[0] =~ m/^eid/i and $tokens[1] and $tokens[1] =~ m/^\d+$/ ) {
-                if ( $tokens[2] and $tokens[2] =~ m/^\[/ ) {
-                    die send_req( $method, "priv/history/eid/$tokens[1]/" . join(' ', @tokens[2..$#tokens]) );
+            if ( @tokens and $tokens[0] =~ m/^eid/i and defined( $tokens[1] ) ) {
+                my $rest_of_tokens = join(' ', @tokens[2..$#tokens]);
+                if ( defined( $rest_of_tokens ) ) {
+                    if ( $rest_of_tokens =~ m/^[[(].*,.*[])]$/ ) {
+                        die send_req( $method, "priv/history/eid/$tokens[1]/$rest_of_tokens" );
+                    }
+                    die send_req( $method, "priv/history/eid/$tokens[1]", $rest_of_tokens );
                 }
-                die send_req( $method, "priv/history/eid/$tokens[1]", join(' ', @tokens[2..$#tokens]) );
+                die send_req( $method, "priv/history/eid/$tokens[1]" );
             }
             
+
             # "/priv/history/nick/:nick [$JSON]
             # "/priv/history/nick/:nick/:tsrange
-            if ( $tokens[0] and $tokens[0] =~ m/^nic/i and $tokens[1] ) {
-                if ( $tokens[2] and $tokens[2] =~ m/^\[/ ) {
-                    die send_req( $method, "priv/history/nick/$tokens[1]/" . join(' ', @tokens[2..$#tokens]) );
+            if ( $tokens[0] and $tokens[0] =~ m/^nic/i and defined( $tokens[1] ) ) {
+                my $rest_of_tokens = join(' ', @tokens[2..$#tokens]);
+                if ( defined( $rest_of_tokens ) ) {
+                    if ( $rest_of_tokens =~ m/^[[(].*,.*[])]$/ ) {
+                        die send_req( $method, "priv/history/nick/$tokens[1]/$rest_of_tokens" );
+                    }
+                    die send_req( $method, "priv/history/nick/$tokens[1]", $rest_of_tokens );
                 }
-                die send_req( $method, "priv/history/nick/$tokens[1]", join(' ', @tokens[2..$#tokens]));
+                die send_req( $method, "priv/history/nick/$tokens[1]" );
             }
 
             # "/priv/history/phid/:phid
@@ -752,11 +866,12 @@ sub parse_tokens {
 
         # "/priv/nick/:nick/?:ts"
         if ( $token =~ m/^nic/i ) {
-            if ( $tokens[1] ) {
-                die send_req( $method, "priv/nick/$tokens[0]/$tokens[1]" );
-            } 
-            if ( $tokens[0] ) {
-                die send_req( $method, "priv/nick/$tokens[0]" );
+            if ( @tokens ) {
+                my $urip = "priv/nick/$tokens[0]";
+                if ( defined $tokens[1] ) {
+                    $urip .= "/" . join( ' ', @tokens[1..$#tokens] );
+                }
+                die send_req( $method, $urip );
             }
         }
 
